" Vim filetype detection script.
" Language:    Haredoc (Hare documentation format)
" Maintainer:  Amelia Clarke <selene@perilune.dev>
" Last Change: 2024-05-12
" Upstream:    https://git.sr.ht/~sircmpwn/hare.vim

if did_filetype()
  finish
endif

let s:cpo_save = &cpo
set cpo&vim

augroup filetypedetect
  au BufNewFile,BufRead README if get(g:, 'filetype_haredoc')
    \ |   if s:IsHareModule('<afile>:h', get(g:, 'haredoc_search_depth', 1))
    \ |     setf haredoc
    \ |   endif
    \ | endif
augroup END

" Recursively search for Hare source files in the current directory and any
" subdirectories, up to a given depth.
function s:IsHareModule(dir, depth)
  if a:depth <= 0
    return !empty(glob(a:dir .. '/*.ha'))
  endif

  return reduce(sort(glob(a:dir .. '/*', v:true, v:true),
    \ { a, b -> isdirectory(a) - isdirectory(b) }),
    \ { acc, n -> acc
    \   || n =~ '\.ha$'
    \   || isdirectory(n)
    \   && s:IsHareModule(n, a:depth - 1) },
    \ v:false)
endfunction

let &cpo = s:cpo_save
unlet s:cpo_save

" vim: et sts=2 sw=2 ts=8
